use std::fmt::{Debug, Formatter};

pub enum PatternParseError {
    UnmatchedOptionalStart,
    UnmatchedOptionalEnd,
    UnmatchedChoiceStart,
    UnmatchedChoiceEnd,
    OptionalInChoice,
    EmptyOptional,
    EmptySequence,
    EmptyChoice,
}
impl Debug for PatternParseError {
    fn fmt(&self, f: &mut Formatter) -> Result<(), std::fmt::Error> {
        let string = match self {
            PatternParseError::UnmatchedOptionalStart => "Missing a ] character",
            PatternParseError::UnmatchedOptionalEnd => "Missing a [ character",
            PatternParseError::UnmatchedChoiceStart => "Missing a ) character",
            PatternParseError::UnmatchedChoiceEnd => "Missing a ( character",
            PatternParseError::OptionalInChoice => "Choice contains an optional pattern",
            PatternParseError::EmptySequence => "Sequence contains no patterns",
            PatternParseError::EmptyOptional => "Optional doesn't contain a pattern",
            PatternParseError::EmptyChoice => "Choice contains no patterns",
        };
        write!(f, "{}", string)
    }
}
