use super::super::{sanitise_input, Word};

pub enum PatternToken {
    Word(Word),
    ChoiceStart,
    ChoiceEnd,
    OptionalStart,
    OptionalEnd,
    ChoiceSeparator,
}
impl PatternToken {
    pub fn tokenise(pattern_string: &str) -> Vec<PatternToken> {
        let mut tokens = Vec::new();
        let mut word = String::new();

        for c in sanitise_input(pattern_string).chars() {
            if "[]()| ".contains(c) && word.len() > 0 {
                tokens.push(PatternToken::Word(Word {
                    word: std::mem::take(&mut word),
                }));
            }
            match c {
                '[' => tokens.push(PatternToken::OptionalStart),
                ']' => tokens.push(PatternToken::OptionalEnd),
                '(' => tokens.push(PatternToken::ChoiceStart),
                ')' => tokens.push(PatternToken::ChoiceEnd),
                '|' => tokens.push(PatternToken::ChoiceSeparator),
                ' ' => (),
                _ => word.push(c),
            }
        }
        if word.len() > 0 {
            tokens.push(PatternToken::Word(Word { word }));
        }
        tokens
    }
}

impl std::fmt::Display for PatternToken {
    fn fmt(&self, f: &mut std::fmt::Formatter) -> Result<(), std::fmt::Error> {
        let string = match self {
            PatternToken::Word(word) => word.as_str(),
            PatternToken::ChoiceStart => "(",
            PatternToken::ChoiceEnd => ")",
            PatternToken::OptionalStart => "[",
            PatternToken::OptionalEnd => "]",
            PatternToken::ChoiceSeparator => "|",
        };
        write!(f, "{}", string)
    }
}
