use super::*;
use crate::Input;

#[test]
#[rustfmt::skip]
fn test_parse_failure() {
    assert!(matches!(Pattern::parse("("), Err(PatternParseError::UnmatchedChoiceStart)));
    assert!(matches!(Pattern::parse(")"), Err(PatternParseError::UnmatchedChoiceEnd)));
    assert!(matches!(Pattern::parse("["), Err(PatternParseError::UnmatchedOptionalStart)));
    assert!(matches!(Pattern::parse("]"), Err(PatternParseError::UnmatchedOptionalEnd)));
    assert!(matches!(Pattern::parse("(WALK"),Err(PatternParseError::UnmatchedChoiceStart)));
    assert!(matches!(Pattern::parse("WALK)"), Err(PatternParseError::UnmatchedChoiceEnd)));
    assert!(matches!(Pattern::parse("[WALK"), Err(PatternParseError::UnmatchedOptionalStart)));
    assert!(matches!(Pattern::parse("WALK]"), Err(PatternParseError::UnmatchedOptionalEnd)));
    assert!(matches!(Pattern::parse("()"), Err(PatternParseError::EmptyChoice)));
    assert!(matches!(Pattern::parse(""), Err(PatternParseError::EmptySequence)));
    assert!(matches!(Pattern::parse("|"), Err(PatternParseError::EmptySequence)));
    assert!(matches!(Pattern::parse("OPEN (A|[THE]) DOOR"), Err(PatternParseError::OptionalInChoice)));
}

#[test]
#[rustfmt::skip]
fn test_parse_success() {
    assert!(Pattern::parse("DOOR").unwrap().to_string() == "DOOR");
    assert!(Pattern::parse("WALK TO DOOR").unwrap().to_string() == "WALK TO DOOR");
    assert!(Pattern::parse("WALK TO [THE] DOOR").unwrap().to_string() == "WALK TO [THE] DOOR");
    assert!(Pattern::parse("(RUN|WALK) TO [THE] DOOR").unwrap().to_string() == "(RUN|WALK) TO [THE] DOOR");
    assert!(Pattern::parse("(|RUN|WALK|) TO [THE] DOOR").unwrap().to_string() == "(RUN|WALK) TO [THE] DOOR");
    assert!(Pattern::parse("(RUN [QUICKLY]|WALK) (AT|TO) [THE] DOOR").unwrap().to_string() == "(RUN [QUICKLY]|WALK) (AT|TO) [THE] DOOR");
    assert!(Pattern::parse("Sprint  ( towards|to) [the]  door ").unwrap().to_string() == "SPRINT (TOWARDS|TO) [THE] DOOR");
    assert!(Pattern::parse("WALK|RUN").unwrap().to_string() == "(WALK|RUN)");
}

#[test]
#[rustfmt::skip]
fn test_parse_success_precise() {
    assert!(format!("{:?}", Pattern::parse("DOOR").unwrap()) == "Word(DOOR)");
    assert!(format!("{:?}", Pattern::parse("OPEN DOOR").unwrap()) == "Sequence(Word(OPEN), Word(DOOR))");
    assert!(format!("{:?}", Pattern::parse("OPEN [THE] DOOR").unwrap()) == "Sequence(Word(OPEN), Optional(Word(THE)), Word(DOOR))");
}

#[test]
#[rustfmt::skip]
fn test_match_failure() {
    assert!(!Pattern::parse("DOOR").unwrap().matches(&Input::parse("DOORS")));
    assert!(!Pattern::parse("DOOR").unwrap().matches(&Input::parse("DOOR KNOB")));
    assert!(!Pattern::parse("OPEN DOOR").unwrap().matches(&Input::parse("OPEN THE DOOR")));
    assert!(!Pattern::parse("OPEN [THE] DOOR").unwrap().matches(&Input::parse("OPEN A DOOR")));
    assert!(!Pattern::parse("OPEN (A|THE) DOOR").unwrap().matches(&Input::parse("OPEN YOUR DOOR")));
}

#[test]
#[rustfmt::skip]
fn test_match_success() {
    assert!(Pattern::parse("DOOR").unwrap().matches(&Input::parse("DOOR")));
    assert!(Pattern::parse("OPEN DOOR").unwrap().matches(&Input::parse("OPEN DOOR")));
    assert!(Pattern::parse("OPEN THE DOOR").unwrap().matches(&Input::parse("OPEN THE DOOR")));
    assert!(Pattern::parse("OPEN [THE] DOOR").unwrap().matches(&Input::parse("OPEN THE DOOR")));
    assert!(Pattern::parse("OPEN [THE] DOOR").unwrap().matches(&Input::parse("OPEN DOOR")));
    assert!(Pattern::parse("OPEN (A|THE) DOOR").unwrap().matches(&Input::parse("OPEN A DOOR")));
    assert!(Pattern::parse("OPEN (A|THE) DOOR").unwrap().matches(&Input::parse("OPEN THE DOOR")));
    assert!(Pattern::parse("OPEN [(A|THE)] DOOR").unwrap().matches(&Input::parse("OPEN DOOR")));
    assert!(Pattern::parse("(RUN FORWARD|(GO|WALK) THROUGH [THE] DOOR)").unwrap().matches(&Input::parse("RUN FORWARD")));
    assert!(Pattern::parse("(RUN FORWARD|(GO|WALK) THROUGH [THE] DOOR)").unwrap().matches(&Input::parse("GO THROUGH THE DOOR")));
    assert!(Pattern::parse("(RUN FORWARD|(GO|WALK) THROUGH [THE] DOOR)").unwrap().matches(&Input::parse("WALK THROUGH DOOR")));
}
