use super::sanitise_input;
use std::fmt::{Display, Formatter};

#[derive(Clone, Eq)]
pub struct Word {
    pub(crate) word: String,
}
impl Word {
    pub fn parse_many(input: &str) -> Vec<Self> {
        sanitise_input(input)
            .split(' ')
            .filter(|word| *word != "")
            .map(|word| Word {
                word: word.to_string(),
            })
            .collect()
    }

    pub fn as_str(&self) -> &str {
        &self.word
    }
}
impl Display for Word {
    fn fmt(&self, f: &mut Formatter) -> Result<(), std::fmt::Error> {
        write!(f, "{}", self.word)
    }
}
impl PartialEq for Word {
    fn eq(&self, other: &Word) -> bool {
        self.word == other.word
    }
}
